@@ -86,6 +86,7 @@ sub GetOptions{
   $options{"global_options"} = {
         boot     => "",
         activate => "bool",
+        color => "",
         timeout  => "",
         default  => "",
         generic_mbr => "bool",
@@ -317,17 +318,22 @@ my $grubdev_pattern = "(?:hd\\d+(?:,\\d+)?|fd\\d+|nd|cd)";
 sub UnixDev2GrubDev {
     my $self = shift;
     my $dev = shift;
+    my $g_dev;
+    local $_;
 
-    unless (defined($dev) and $dev) {
+    if ($dev eq "") {
 	$self->l_error ("GRUB::UnixDev2GrubDev: Empty device to translate");
 	return ""; # return an error
     }
+
     # Seems to be a grub device already 
     if ($dev =~ /^\(${grubdev_pattern}\)$/) {
 	$self->l_warning ("GRUB::UnixDev2GrubDev: Not translating device $dev");
 	return $dev;
     }
 
+    $self->l_milestone ("GRUB::UnixDev2GrubDev: Translating $dev ...");
+
     # remove parenthesis to be able to handle entries like "(/dev/sda1)" which
     # might be there by error
     $dev =~ s/^\((.*)\)$/$1/;
@@ -335,84 +341,78 @@ sub UnixDev2GrubDev {
     # This gives me the devicename, whether $dev is the device or a link!
     # This works for udev (kernel) devices only, devicemapper doesn't 
     # need to be changed here 
+
     my $original = $dev;
     my $kernel_dev = $self->GetKernelDevice($dev);
 
     $self->l_milestone ("GRUB::UnixDev2GrubDev: kernel device: $kernel_dev");
 
     my $partition = undef;
+
     if ($kernel_dev =~ m#/dev/md\d+#) {
 	my @members = @{$self->MD2Members ($kernel_dev) || []};
 	# FIXME! This only works for mirroring (Raid1)
-	$kernel_dev = $members[0] || $kernel_dev;
-	$self->l_milestone ("GRUB::UnixDev2GrubDev: First device of MDRaid: $original --> $kernel_dev");
+	$kernel_dev = $self->GetKernelDevice($members[0] || $kernel_dev );
+	$self->l_milestone ("GRUB::UnixDev2GrubDev: First device of MDRaid: $original -> $kernel_dev");
+    }
+
+    # print all entries of device.map - for debugging
+    if ($self->{device_map}) {
+        $self->l_milestone ("GRUB::UnixDev2UnixDev: device_map:");
+
+	for (sort keys %{$self->{device_map}}) {
+	    $self->l_milestone ("unix device: $_ <==> grub device: $self->{device_map}{$_}");
+	}
+    }
+    else {
+	$self->l_warning ("GRUB::UnixDev2GrubDev: empty device_map");
     }
 
     # fetch the underlying device (sda1 --> sda)
-    foreach my $dev_ref (@{$self->{"partitions"}}) {
-	if ($dev_ref->[0] eq $kernel_dev) {
-	    $kernel_dev = $dev_ref->[1];
-	    $partition = $dev_ref->[2] - 1;
-	    $self->l_milestone ("GRUB::UnixDev2GrubDev: dev_ref: ".$dev_ref->[0]." ".$dev_ref->[1]." ".$dev_ref->[2]);
+    for (@{$self->{partitions}}) {
+	if ($_->[0] eq $kernel_dev) {
+	    $kernel_dev = $_->[1];
+	    $partition = $_->[2] - 1;
+	    $self->l_milestone ("GRUB::UnixDev2GrubDev: dev base part: $_->[0] $_->[1] $_->[2]");
 	    last;
 	}
     }
 
-    for my $map_dev (keys  %{$self->{"device_map"}}){
-      if ($kernel_dev eq $self->GetKernelDevice($map_dev)){
-        $dev =  $self->{"device_map"}->{$map_dev};
-        values %{$self->{"device_map"}}; #reset hash iterator
-        return "($dev)" if ( $kernel_dev eq $original ); #disk dev, no partition
-        last;
-      }
+    for (sort keys %{$self->{device_map}}) {
+        if ($kernel_dev eq $self->GetKernelDevice($_)) {
+            $g_dev = $self->{device_map}{$_};
+            last;
+        }
     }
 
-    # print all entries of device.map. This is rather for debugging
-    if (exists $self->{"device_map"}) {
-        $self->l_milestone ("GRUB::UnixDev2UnixDev: device_map: ".$self->{"device_map"});
-	$self->l_milestone ("GRUB::UnixDev2GrubDev: Read from internal structure device_map:");
+    # fallback if translation has failed - this is good enough for many cases
+    # FIXME: this is nonsense - we should return an error
+    if (!$g_dev) {
+        $self->l_warning("GRUB::UnixDev2GrubDev: Unknown device/partition, using fallback");
 
-	while ((my $unix_dev, my $grub_dev) = each (%{$self->{"device_map"}})) {
-	    $self->l_milestone ("unix device: $unix_dev <==> grub device: $grub_dev");
-	}
+        $g_dev = "hd0";
+        $partition = undef;
+
+        if (
+            $original =~ m#^/dev/(?:vx|das|[ehpsvx])d[a-z]{1,2}(\d+)$# ||
+            $original =~ m#^/dev/\S+\-part(\d+)$# ||
+            $original =~ m#^/dev(?:/[^/]+){1,2}p(\d+)$#
+        ) {
+            $partition = $1 - 1;
+            $partition = 0 if $partition < 0;
+        }
+    }
+
+    if (defined $partition) {
+        $g_dev = "($g_dev,$partition)";
+        $self->l_milestone ("GRUB::UnixDev2GrubDev: Translated UNIX partition -> GRUB device: $original to $g_dev");
     }
     else {
-	$self->l_warning ("GRUB::UnixDev2GrubDev: Internal structure device_map doesn't exist.");
+        $g_dev = "($g_dev)";
+        $self->l_milestone ("GRUB::UnixDev2GrubDev: Translated UNIX device -> GRUB device: $original to $g_dev");
     }
 
-    $self->l_milestone ("GRUB::UnixDev2GrubDev: Translated UNIX device/partition -> GRUB device: $original to $dev");
-
-    # fallback to grub device hd0 if translation has failed - this is good
-    # enough for many cases
-    my $partition_fallback = 0;
-    
-    if ( $original =~ m/(\d+)\s*$/ )
-    {
-        $partition_fallback = $1 - 1;
-        $partition_fallback = 0 if ($partition_fallback < 0 );
-    }
-
-    if ( $original =~ m/(\d+)\s*$/ )
-    {
-       $partition_fallback = $1 - 1;
-       $partition_fallback = 0 if ($partition_fallback < 0 );
-    }
-
-    if ($dev !~ /^${grubdev_pattern}$/) {
-	$dev = "hd0";
-	$self->l_warning ("GRUB::UnixDev2GrubDev: Unknown device '$original', fall back to ($dev)");
-    }
-
-    $dev = defined ($partition)
-	? "($dev,$partition)"
-	: "($dev,$partition_fallback)";
-
-    $self->l_warning("GRUB::UnixDev2GrubDev: Unknown partition, fallback to first")
-      unless (defined ($partition));
-
-    $self->l_milestone ("GRUB::UnixDev2GrubDev: Translated UNIX->GRUB: $original to $dev");
-
-    return $dev;
+     return $g_dev;
 }
 
 =item
@@ -621,6 +621,10 @@ sub ParseLines {
     };
     $self->l_milestone ("GRUB::Parselines: avoided_reading device map.") if ($avoid_reading_device_map );
     $self->{"device_map"} = \%devmap	if (! $avoid_reading_device_map);
+    $self->l_milestone ("GRUB::Parselines: device_map: ".$self->{"device_map"});
+    while ((my $unix, my $fw) = each (%{$self->{"device_map"}})) {
+        $self->l_milestone ("GRUB::Parselines: device_map: $unix <-> $fw.");
+    }
 
     # and now proceed with menu.lst
     my @menu_lst = @{$files{Bootloader::Path::Grub_menulst()} || []};
@@ -782,15 +786,6 @@ sub CreateLines {
     my @device_map = ();
     while ((my $unix, my $fw) = each (%{$self->{"device_map"}}))
     {
-        #multipath handling, multipath need real device, because multipath
-        # device have broken geometry (bnc #448110)
-        if ( defined $self->{"multipath"} ){
-          while ((my $phys, my $mp) = each (%{$self->{"multipath"}})){
-            if ( $mp eq $self->GetKernelDevice($unix) ) {
-              $unix = $phys;
-            }
-          }
-        }
 	my $line = "($fw)\t$unix";
 	push @device_map, $line;
     }
@@ -1558,7 +1553,7 @@ sub Info2Section {
       $vga = "vga=mode-$vga " if $vga ne "";
       my $value = "$xen $vga$append";
       push @lines, {
-	"key" => "kernel",
+      	"key" => "kernel",
         "value" => $value,
       };
     }
@@ -2020,7 +2015,7 @@ sub GrubDev2MountPoint {
 	    $self->l_debug ("GRUB::GrubDev2MountPoint : record $mp <-> $d");
 	    if ($self->GetKernelDevice($d) eq $self->GetKernelDevice($device)) { 
 	      $self->l_milestone ("GRUB::GrubDev2MountPoint : find mountpoint: $mp");
-    		$mountpoint = $mp;
+    	  $mountpoint = $mp;
 	    }
 	}
     }
