@@ -66,6 +66,8 @@ C<< Bootloader::Tools::AdjustSectionNameAppendix ($mode, $sect_ref_new, $sect_re
 
 C<< $exec_with_path = Bootloader::Tools::AddPathToExecutable($executable); >>
 
+C<< Bootloader::Tools::DeviceMapperActive (); >>
+
 =head1 DESCRIPTION
 
 =over 2
@@ -80,7 +82,7 @@ use base 'Exporter';
 
 our @EXPORT = qw(InitLibrary CountImageSections CountSections
 		 RemoveImageSections GetDefaultImage
-		 GetDefaultInitrd GetBootloader UpdateBootloader
+		 GetDefaultInitrd GetBootloader UpdateBootloader GetFittingSection
 		 GetGlobals SetGlobals 
 		 GetSectionList GetSection 
 		 AddSection RemoveSections
@@ -95,6 +97,7 @@ my $lib_ref = undef;
 my $dmsetup = undef;
 my $mdadm = undef;
 my $multipath = undef;
+my $devicemapper = undef;
 
 sub DumpLog {
     my $core_lib = shift;
@@ -171,7 +174,7 @@ sub ResolveCrossDeviceSymlinks {
     my $path = shift;
 
     my $core_lib = Bootloader::Core->new ();
-    $path = $core_lib->RealFileName ($path);
+    $path = $core_lib->ResolveCrossDeviceSymlinks ($path);
 
     DumpLog ($core_lib);
     return $path;
@@ -245,7 +248,7 @@ sub ReadPartitions {
     } readdir(BLOCK_DEVICES);
     closedir BLOCK_DEVICES;
 
-    $logger->milestone("ReadPartitions: Finded disks: ". join (",",@disks));
+    $logger->milestone("ReadPartitions: Found disks: ". join (",",@disks));
 
     # get partition info for all partitions on all @disks
     my @devices = ();
@@ -270,7 +273,7 @@ sub ReadPartitions {
 	    } readdir (BLOCK_DEVICES);
 	    closedir BLOCK_DEVICES;
 
-            $logger->milestone("ReadPartitions: Finded parts: ". join (",",@parts));
+            $logger->milestone("ReadPartitions: Found parts: ". join (",",@parts));
 
 	    # generate proper device names and other info for all @part[ition]s
       #raid have ! in names for /dev/raid/name (bnc#607852)
@@ -386,6 +389,7 @@ Gets multipath configuration. Return reference to hash map, empty if system does
 
 sub GetUdevMapping {
   my %mapping= ();
+  local $_;
 
   my @output = `find -P /dev -type b`;
   chomp @output;
@@ -415,23 +419,33 @@ sub GetUdevMapping {
       $logger->error("UDEVMAPPING: dmdev $dev doesn't have defined DM_NAME in udev") unless defined $dmdev;
       my $prevdev = $dev;
       $dev = "/dev/mapper/$dmdev";
-      $dev = $dev."_part$dmpart" if defined $dmpart;
+      # DM_NAME could contain also part in some case (bnc#590637)
+      $dev = $dev."_part$dmpart" if (defined $dmpart and $dev !~ /_part[0-9]+$/);
       $mapping{$prevdev} = $dev; #maps also dm dev to device mapper
     } #end of workaround
 
-    for my $line (@output2)
-    {
-      if ($line =~ m/S:\s(.*)$/)
-      {
-        $mapping{"/dev/$1"} = $dev;
-      }
+    for (@output2) {
+      $mapping{"/dev/$1"} = $dev if /S:\s(.*)$/;
     }
   }
 
   while (my ($k,$v) = each (%mapping)){
-      $logger->milestone ("UDEV MAPPING: ".($k||"")." -> ".($v||""));
+    $logger->milestone("UDEV MAPPING: ".($k||"")." -> ".($v||""));
   }
 
+  if(!(keys %mapping)) {
+    $logger->milestone("*** WARNING: No UDEV mapping! ***");
+    $logger->milestone("device tree:");
+    for (`find -P /dev -type b -ls 2>/dev/null`) {
+      chomp;
+      $logger->milestone("  $_");
+    }
+    $logger->milestone("udevinfo:");
+    for (`udevadm info -e 2>&1`) {
+      chomp;
+      $logger->milestone("  $_");
+    }
+  }
 
   return \%mapping;
 }
@@ -765,6 +779,8 @@ sub InitLibrary {
     $lib_ref->ReadSettings();
 
     DumpLog ($lib_ref->{"loader"});
+
+    return $lib_ref;
 }
 
 
@@ -784,11 +800,11 @@ sub match_section {
 	$core_lib->l_milestone ("Tools::match_section: matching key: $opt");
 	if ($opt eq "image" or $opt eq "initrd") {
 	    $match = (ResolveCrossDeviceSymlinks($sect_ref->{"$opt"}) eq
-		      $opt_ref->{"$opt"});
+		      ResolveCrossDeviceSymlinks($opt_ref->{"$opt"}));
 	    # Print info for this match
 	    $core_lib->l_milestone ("Tools::match_section: key: $opt, matched: " .
 		ResolveCrossDeviceSymlinks($sect_ref->{"$opt"}) .
-		", with: " . $opt_ref->{"$opt"} . ", result: $match");
+		", with: " . ResolveCrossDeviceSymlinks($opt_ref->{"$opt"}) . ", result: $match");
 	}
 	else {
 	    $match = ($sect_ref->{"$opt"} eq $opt_ref->{"$opt"});
@@ -1064,6 +1080,54 @@ sub GetSection {
     return undef;
 }
 
+=item
+C<< Bootloader::Tools::GetFittingSection($name, $image, $type, \@sections); >>
+
+Fing fitting section for given name and image type.
+If not find return undef.
+
+=cut
+
+sub GetFittingSection {
+  my $name = shift;
+  my $image = shift;
+  my $type = shift;
+  my $sections = shift;
+  my $ret = undef;
+
+
+  if ($image !~ m/^.*-([a-zA-Z0-9]+)(\..*)?$/) {
+    return undef;
+  }
+  my $flavor = $1;
+  my $fallback = ($name =~ m/failsafe/i);
+ 
+  # Heuristic one - try find same flavor
+  foreach my $s (@{$sections}) {
+    next unless $s->{"type"} eq $type;
+    if (defined ($s->{"image"}) && $s->{"image"} =~ m/$flavor/) {
+      my $sec_fallback = (defined $s->{"original_name"} && $s->{"original_name"} =~ m/failsafe/i) ||
+                         (defined $s->{"name"} && $s->{"name"} =~ m/failsafe/i);
+      next if ($sec_fallback xor $fallback); #skip if they are not same failsafe or non-failsafe
+      $ret = $s;
+      last;
+    }
+  }
+
+  return $ret if $ret;
+
+  # Heuristic two - first fitting section with same type
+  foreach my $s (@{$sections}) {
+    next unless $s->{"type"} eq $type;
+    my $sec_fallback = (defined $s->{"original_name"} && $s->{"original_name"} =~ m/failsafe/i) ||
+                       (defined $s->{"name"} && $s->{"name"} =~ m/failsafe/i);
+    next if ($sec_fallback xor $fallback); #skip if they are not same failsafe or non-failsafe
+    $ret = $s;
+    last;
+  }
+
+  return $ret;
+}
 
 =item
 C<< Bootloader::Tools::AddSection($name, @params); >>
@@ -1094,27 +1158,68 @@ sub AddSection {
     my %def = ();
 
     my @sections = @{$lib_ref->GetSections ()};
+    my $core_lib = $lib_ref->{"loader"};
 
-    # FIXME: sf@: what is this code good for?
-    # FIXME: removed resetting root parameter if it's already set
-    foreach my $s (@sections) {
-	if (defined ($s->{"initial"}) && $s->{"initial"}) {
-	    %def = %{$s};
-	    last;
-	}
-    }
+    my $fitting_section = GetFittingSection($name,$option{"image"},$option{"type"},\@sections);
 
-    while ((my $k, my $v) = each (%def)) {
-	if (substr ($k, 0, 2) ne "__" && $k ne "original_name"
-		&& $k ne "initrd") {
-	    if (!defined $new{$k}) {
-		$new{$k} = $v;
-	    }	
-	}
+    if ($fitting_section) {
+      %def = %{$fitting_section};
+      $core_lib->l_milestone ("Tools::AddSection: Fitting section found so use it");
+      while ((my $k, my $v) = each (%def)) {
+        if (substr ($k, 0, 2) ne "__" && $k ne "original_name"
+        		&& $k ne "initrd") {
+          $new{$k} = $v;
+      	}
+      }
+
+      if (exists $option{"image"}) {
+        if (exists $new{"kernel"}) { 
+    	    $new{"kernel"} = delete $option{"image"};
+        }	else {	
+    	    $new{"image"} = delete $option{"image"};
+      	}
+      }
+      } else {
+        $core_lib->l_milestone ("Tools::AddSection: Fitting section not found. Use sysconfig values as fallback.");
+        my $sysconf;
+        if ($name =~ m/^Failsafe.*$/ or $option{"original_name"} eq "failsafe") {
+          $sysconf =  GetSysconfigValue("FAILSAFE_APPEND");
+          $new{"append"} = $sysconf if (defined $sysconf);
+          $sysconf = GetSysconfigValue("FAILSAFE_VGA");
+          $new{"vgamode"} = $sysconf if (defined $sysconf);
+          $sysconf = GetSysconfigValue("IMAGEPCR");
+        }
+        elsif ($option{"type"} eq "xen") 
+        {
+          $sysconf = GetSysconfigValue("XEN_KERNEL_APPEND");
+          $new{"append"} = $sysconf if (defined $sysconf);
+          $sysconf =  GetSysconfigValue("XEN_VGA");
+          $new{"vgamode"} = $sysconf if (defined $sysconf);
+          $sysconf =  GetSysconfigValue("XEN_APPEND");
+          $new{"xen_append"} =  $sysconf if (defined $sysconf);
+        }
+        else 
+        {
+          #RT kernel have special args bnc #450153
+          if ($new{"image"} =~ m/-rt$/ && defined (GetSysconfigValue("RT_APPEND"))){
+            $sysconf = GetSysconfigValue("RT_APPEND");
+            $new{"append"} = $sysconf if (defined $sysconf);
+            $sysconf = GetSysconfigValue("RT_VGA");
+            $new{"vgamode"} = $sysconf if (defined $sysconf);
+          } else {
+            $sysconf = GetSysconfigValue("DEFAULT_APPEND");
+            $new{"append"} = $sysconf if (defined $sysconf);
+            $sysconf = GetSysconfigValue("DEFAULT_VGA");
+            $new{"vgamode"} = $sysconf if (defined $sysconf);
+          }
+      }
+
+      $sysconf = GetSysconfigValue("CONSOLE");
+      $new{"console"} = $sysconf if (defined $sysconf);
     }
 
     foreach (keys %option) {
-	$new{"$_"} = $option{"$_"};
+      $new{"$_"} = $option{"$_"};
     }
     $new{"name"} = $name;
 
@@ -1122,72 +1227,11 @@ sub AddSection {
     AdjustSectionNameAppendix ("add", \%new);
 
     my $failsafe_modified = 0;
-
-
-
-    # FIXME: Failsafe parameters should be set dynamically in the future
-    if ($name =~ m/^Failsafe.*$/) {
-	my $arch = `uname --hardware-platform`;
-	chomp ($arch);
-
-	if ($arch eq "i386") {
-	    $new{"append"} = "showopts ide=nodma apm=off acpi=off noresume nosmp noapic maxcpus=0 edd=off x11failsafe nomodeset";
-	}
-	elsif ($arch eq "x86_64") {
-	    $new{"append"} = "showopts ide=nodma apm=off acpi=off noresume edd=off x11failsafe nomodeset";
-	}
-	elsif ($arch eq "ia64") {
-	    $new{"append"} = "ide=nodma nohalt noresume 3";
-	}
-	elsif ($arch eq "s390x") {
-	    $new{"append"} = "";
-  }
-	else {
-	    print ("Architecture $arch does not support failsafe entries.\n");
-	}
-
-	$failsafe_modified = 1;
-
-	# Don't make the failsafe entry the default one
-	$default = 0;
-    }
-
-    my $sysconf;
     if ($name =~ m/^Failsafe.*$/ or $option{"original_name"} eq "failsafe") {
-        $sysconf =  GetSysconfigValue("FAILSAFE_APPEND");
-        $new{"append"} = $sysconf if (defined $sysconf);
-        $sysconf = GetSysconfigValue("FAILSAFE_VGA");
-        $new{"vgamode"} = $sysconf if (defined $sysconf);
-        $failsafe_modified = 1;
-	$default = 0;
+      $failsafe_modified = 1;
+	    $default = 0;
     }
-    elsif ($option{"type"} eq "xen") 
-    {
-        $sysconf = GetSysconfigValue("XEN_KERNEL_APPEND");
-        $new{"append"} = $sysconf if (defined $sysconf);
-        $sysconf =  GetSysconfigValue("XEN_VGA");
-        $new{"vgamode"} = $sysconf if (defined $sysconf);
-        $sysconf =  GetSysconfigValue("XEN_APPEND");
-        $new{"xen_append"} =  $sysconf if (defined $sysconf);
-    }
-    else 
-    {
-        #RT kernel have special args bnc #450153
-        if ($new{"image"} =~ m/-rt$/ && defined (GetSysconfigValue("RT_APPEND"))){
-          $sysconf = GetSysconfigValue("RT_APPEND");
-          $new{"append"} = $sysconf if (defined $sysconf);
-          $sysconf = GetSysconfigValue("RT_VGA");
-          $new{"vgamode"} = $sysconf if (defined $sysconf);
-        } else {
-          $sysconf = GetSysconfigValue("DEFAULT_APPEND");
-          $new{"append"} = $sysconf if (defined $sysconf);
-          $sysconf = GetSysconfigValue("DEFAULT_VGA");
-          $new{"vgamode"} = $sysconf if (defined $sysconf);
-        }
-     }
 
-    $sysconf = GetSysconfigValue("CONSOLE");
-    $new{"console"} = $sysconf if (defined $sysconf);
     $new{"__modified"} = 1;
     $new{"root"} = $lib_ref->GetMountPoints()->{"/"};
 
@@ -1214,8 +1258,6 @@ sub AddSection {
 	}
     }
 
-    my $core_lib = $lib_ref->{"loader"};
-
     # Print new section to be added to logfile
     $core_lib->l_milestone ("Tools::AddSection: New section to be added :\n\n' " .
 			join("'\n' ",
