@@ -25,6 +25,8 @@ BEGIN {
         getNfsInfo => ["function", "integer", "any", ["&list", "any"]],
         getLoopInfo => ["function", "integer", "any", ["&list", "any"]],
         getDmInfo => ["function", "integer", "any", ["&list", "any"]],
+        getBtrfsInfo => ["function", "integer", "any", ["&list", "any"]],
+        getTmpfsInfo => ["function", "integer", "any", ["&list", "any"]],
         getDmraidInfo => ["function", "integer", "any", "&string", ["&list", "any"]],
         getDmmultipathInfo => ["function", "integer", "any", "&string", ["&list", "any"]],
         getFsCapabilities => ["function", "boolean", "any", "integer", "any"],
@@ -88,6 +90,8 @@ BEGIN {
         getDefaultMountBy => ["function", "integer", "any"],
         setDefaultFs => ["function", "void", "any", "integer"],
         getDefaultFs => ["function", "integer", "any"],
+        setDefaultSubvolName => ["function", "void", "any", "&string"],
+        getDefaultSubvolName => ["function", "string", "any"],
         getEfiBoot => ["function", "boolean", "any"],
         setRootPrefix => ["function", "void", "any", "&string"],
         getRootPrefix => ["function", "string", "any"],
@@ -127,6 +131,13 @@ BEGIN {
         modifyFileLoop => ["function", "integer", "any", "&string", "&string", "boolean", "integer"],
         removeFileLoop => ["function", "integer", "any", "&string", "boolean"],
         removeDmraid => ["function", "integer", "any", "&string"],
+        existSubvolume => ["function", "boolean", "any", "&string", "&string"],
+        createSubvolume => ["function", "integer", "any", "&string", "&string"],
+        removeSubvolume => ["function", "integer", "any", "&string", "&string"],
+        extendBtrfsVolume => ["function", "integer", "any", "&string", ["&list", "string"]],
+        shrinkBtrfsVolume => ["function", "integer", "any", "&string", ["&list", "string"]],
+        addTmpfsMount => ["function", "integer", "any", "&string", "&string"],
+        removeTmpfsMount => ["function", "integer", "any", "&string"],
         getCommitInfos => ["function", "void", "any", ["&list", "any"]],
         getLastAction => ["function", "string", "any"],
         getExtendedErrorMessage => ["function", "string", "any"],
