@@ -4,7 +4,7 @@
 # Package:	Configuration of users and groups
 # Summary:	module for first stage user configuration
 #
-# $Id: UsersSimple.pm 61042 2010-02-26 13:48:03Z jsuchome $
+# $Id: UsersSimple.pm 65903 2011-09-22 09:04:32Z jsuchome $
 #
 
 package UsersSimple;
@@ -47,7 +47,7 @@ my @users			= ();
 
 
 # password encryption method
-my $encryption_method		= "blowfish";
+my $encryption_method		= "sha512";
 
 # mail alias for root
 my $root_alias			= "";
@@ -70,6 +70,8 @@ my %max_lengths			= (
     "des"	=> 8,
     "md5"	=> 127,
     "blowfish"	=> 72,
+    "sha256"	=> 127, # arbitrary high number, there's probably no limit
+    "sha512"	=> 127
 );
 
 # name of user that should be logged in automatically
@@ -641,7 +643,7 @@ sub CheckObscurity {
     $filtered 		= reverse $pw;
     if ($filtered eq $pw) {
 	# popup question
-        return __("You have used a palindrom for the password.");
+        return __("You have used a palindrome for the password.");
     }
 
     # check for numbers
@@ -918,12 +920,16 @@ sub Write {
     }
     # enable inst_user to either run auth client or write first user
     if ($after_auth ne "users" || $user_defined) {
-	y2milestone ("enabling step 'user' for second stage");
+	# do not let both steps to run (bnc#672139)
 	if ($user_defined) {
+	    y2milestone ("enabling 'user_non_interactive', disabling 'user' for second stage");
 	    ProductControl->EnableModule ("user_non_interactive");
+	    ProductControl->DisableModule ("user");
 	}
 	else {
+	    y2milestone ("enabling 'user', disabling 'user_non_interactive' for second stage");
 	    ProductControl->EnableModule ("user");
+	    ProductControl->DisableModule ("user_non_interactive");
 	}
     }
     # no user entered + 2nd stage visible => enable clients (bnc#393722)
@@ -945,7 +951,9 @@ sub Read {
     my $file	= Directory->vardir()."/users_first_stage.ycp";
     my $ret	= 0;
 
-    if (FileUtils->Exists ($file) && ($force || $first_stage_data_not_read)) {
+    if (Stage->cont () && FileUtils->Exists ($file) &&
+	($force || $first_stage_data_not_read)) 
+    {
 	my $data	= SCR->Read (".target.ycp", $file);
 	$first_stage_data_not_read	= 0;
 	if (defined $data && ref ($data) eq "HASH") {
